@@ -55,7 +55,7 @@ print.yai = function(x,...)
    else
    {
       nPr=min(10,nrow(x$neiDstTrgs))
-      part=data.frame(x$neiDstTrgs[1:nPr,],x$neiIdsTrgs[1:nPr,],
+      part=data.frame(x$neiDstTrgs[1:nPr,,drop=FALSE],x$neiIdsTrgs[1:nPr,,drop=FALSE],
                       stringsAsFactors = FALSE)
       names(part)=c(colnames(x$neiDstTrgs),colnames(x$neiIdsTrgs))
       cat ("First",nPr,"targets:\n")
@@ -65,7 +65,7 @@ print.yai = function(x,...)
    else
    {
       nPr=min(10,nrow(x$neiDstRefs))
-      part=data.frame(x$neiDstRefs[1:nPr,],x$neiIdsRefs[1:nPr,],
+      part=data.frame(x$neiDstRefs[1:nPr,,drop=FALSE],x$neiIdsRefs[1:nPr,,drop=FALSE],
                       stringsAsFactors = FALSE)
       names(part)=c(colnames(x$neiDstRefs),colnames(x$neiIdsRefs))
       cat ("First",nPr,"references:\n")
