@@ -167,16 +167,18 @@ ANNbool ANNorthRect::inside(int dim, ANNpoint p)
 //	Error handler
 //----------------------------------------------------------------------
 
-void annError(char *msg, ANNerr level)
+void annError(const char *msg, ANNerr level)
 {
 	if (level == ANNabort) {
 	  //cerr << "ANN: ERROR------->" << msg << "<-------------ERROR\n";
-	  Rprintf("ANN: ERROR");
-	  std::exit(1);
+	  Rprintf("ANN Fatal ERROR:");
+	  Rprintf(msg);
+//	  std::exit(1);
 	}
 	else {
 	  //cerr << "ANN: WARNING----->" << msg << "<-------------WARNING\n";
-	  Rprintf("ANN: ERROR");
+	  Rprintf("ANN WARNING:");
+	  Rprintf(msg);
 	}
 }
 
